/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamchannel.h"

/* From gdamchannel.c */
extern const char* default_addr_spec;
extern GdamBuffer default_auth_info;
static gboolean auth_inited = FALSE;

const char* gdam_channel_usage_message = 
	"\t--gdam-server host:port            Location of the gdam server.\n"
	"\t--device device_name               Name of the device.\n"
	"\t--debug                            Turn on debugging.\n"
	;
	

gboolean gdam_channel_init(int* argc,
                           char*** argv,
			   GdamChannel** channel_out)
{
	int i, out_at;

	gdam_buffer_construct(&default_auth_info);
	auth_inited = FALSE;
	*channel_out = NULL;

	gdam_arg_init();

	for (i = 1; i < *argc; i++) {
		const char* arg = (*argv)[i];

		if (strcmp(arg, "--gdam-server") == 0) {
			(*argv)[i] = NULL;
			default_addr_spec = (*argv)[++i];
			(*argv)[i] = NULL;
			if (!default_addr_spec) {
				g_warning("--gdam-server must be follow by "
					  "a server location");
				return FALSE;
			}
			continue;
		}

		if (strcmp(arg, "--debug") == 0) {
			gdam_debugging_on = TRUE;
			(*argv)[i] = NULL;
			continue;
		}

		if (strcmp(arg, "--") == 0)
			break;
	}
	if (!auth_inited) {
		gdam_buffer_construct(&default_auth_info);
		gdam_buffer_prepend_length(&default_auth_info);
		auth_inited = TRUE;
	}

	/* Condense deleted arguments. */
	for (i = out_at = 1; i < *argc; i++)
		if ((*argv)[i] != NULL) {
			if (out_at == i)
				out_at++;
			else
				(*argv)[out_at++] = (*argv)[i];
		}
	*argc = out_at;

	*channel_out = gdam_channel_new();
	g_return_val_if_fail(*channel_out != NULL, FALSE);
	gdam_channel_attach(*channel_out);
	(*channel_out)->quit_main = 1;

#if 0
	if (gdam_channel_get_server_side_int(*channel_out,
	                                     "devices",
					     device_name,
					     device_id)) {
		gdam_debug("using opened device %d", *device_id);
	} else {
		/* Create the device... */
		*device_id = gdam_channel_create_sink(*channel_out,
						      device_name);
	}
	if (!*device_id) {
		g_warning("couldn't create device");
		gdam_channel_destroy(*channel_out);
		return FALSE;
	}
	if (gdam_channel_get_server_side_int(*channel_out,
	                                     "device-feeds",
					     device_name,
					     mixer_id)) {
		gdam_debug("using old mixer %d", *mixer_id);
	} else {
		*mixer_id = gdam_channel_create_source_generic(
				*channel_out,
				"GdamMix",
				1,
				"terminates",
				GDAM_ARG_TYPE_INT,
				FALSE);

		if (!gdam_channel_feed_sink(*channel_out,
		                            *device_id,
					    *mixer_id)) {
			g_warning("error feeding sink");
			return 1;
		}
	}
#endif

	return TRUE;
}
