/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Mike Radford, Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
	mrad@blorf.com <Mike Radford>
*/
#ifndef __GDAM_SOUNDSOURCE_COMPRESSING_MIX_H_
#define __GDAM_SOUNDSOURCE_COMPRESSING_MIX_H_

#include <glib.h>
#include <gdam/gdamsource.h>
#include <gdam/gdambuffer.h>	/* for GdamTime */

GtkType gdam_compressing_mix_get_type();
#define GDAM_TYPE_COMPRESSING_MIX              (gdam_compressing_mix_get_type ())
#define GDAM_COMPRESSING_MIX(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_COMPRESSING_MIX, GdamCompressingMix))
#define GDAM_COMPRESSING_MIX_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_COMPRESSING_MIX, GdamCompressingMixClass))
#define GDAM_IS_COMPRESSING_MIX(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_COMPRESSING_MIX))
#define GDAM_IS_COMPRESSING_MIX_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_COMPRESSING_MIX))

typedef struct _GdamCompressingMix GdamCompressingMix;
typedef struct _GdamCompressingMixClass GdamCompressingMixClass;
typedef struct _GdamCompressingMixElement GdamCompressingMixElement;

struct _GdamCompressingMixElement {
	/* The next sample (in the mix's perception of time)
	 * to fill from the subsource. */
	GdamTime		read_to;

	/* The source to read from. */
	GdamSource*		sub_source;

	/* We use an inplace list, b/c we want deleting
	 * a GdamCompressingMixElement to be fast... */
	GdamCompressingMixElement*		next;
	GdamCompressingMixElement*		prev;

	/* Passed to the init_copy and augment functions. */
	gpointer                source_info;

	unsigned		flags;
};

/* For GdamCompressingMixElement::flags. */
#define MIX_ELEMENT_IN_FUTURE		1

struct _GdamCompressingMix {
	GdamSource		base;

	GdamCompressingMixElement*		first_source;
	GdamCompressingMixElement*		last_source;

	/* A circular buffer we are filling up. */
	int			buffer_size;
	int*			buffer;		/* left-right sample data */
	int			buffer_data_start;

	/* A temporary buffer, the same length as `buffer' (ie buffer_size)
	 * that is used only during `read'.  Just saves
	 * us a `malloc' during read... */
	GdamSample*		tmp_buf;

	/* Keeps track of how many samples in the buffer have been
	 * initialized. */
	GdamTime 		cleared_to;

	GdamTime		samples_output;
	unsigned		terminates : 1;
};

struct _GdamCompressingMixClass {
	GdamSourceClass klass;
	void        (*connect)(GdamCompressingMix*,
	                       GdamSource* to_add, gint64 when);

	void        (*compress)(GdamCompressingMix* mix,
				GdamSample*         compressed,
	                        int*                uncompressed,
				int                 num_samples);

};


GdamCompressingMix*       gdam_compressing_mix_new(gboolean terminates_when_done);

/* If `when' is `-1' it means `immediately'. */
void           gdam_compressing_mix_connect         (GdamCompressingMix*    mix,
                                         GdamSource* sub_source,
				         GdamTime    when);

GdamTime       gdam_compressing_mix_get_pos         (GdamCompressingMix*    mix);
int            gdam_compressing_mix_num_sources     (GdamCompressingMix*    mix);
#endif
